#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>

#include "window.h"
#include "cardfile.h"



/*
  CardfileWndProc - window procedure for the main window
*/
long PASCAL CardfileWndProc(hWnd, message, wParam, lParam)
  HWND hWnd;
  WORD message;
  WORD wParam;
  DWORD lParam;
{
  INT  rc;
  char buf[256];

  switch (message)
  {
    case WM_COMMAND    :
      switch (wParam)
      {
	case ID_NEW         :
          if (CardFileMaybeSave(ID_NEW) == FALSE)
            break;
          szCurrFileName[0] = '\0';
          memset((char *) &CurrCardFile, 0, sizeof(CurrCardFile));
          ListFree(&CurrRecordList, TRUE);
          break;

	case ID_OPEN        : 
          if (CardFileMaybeSave(ID_OPEN) == FALSE)
            break;
          szCurrFileName[0] = '\0';
          memset((char *) &CurrCardFile, 0, sizeof(CurrCardFile));
          ListFree(&CurrRecordList, TRUE);
          if (DlgOpenFile(NULLHWND, "*.crd", buf))
            CardFileOpen(buf, ID_OPEN);
          break;

	case ID_SAVE        :
          CardFileMaybeSave(ID_SAVE);
          break;

	case ID_MERGE       :
          if (DlgOpenFile(NULLHWND, "*.crd", buf))
            CardFileOpen(buf, ID_MERGE);
          break;

	case ID_PRINT       :
          CardPrint(CurrCardFile.iCurrRecord, (FILE *) NULL);
          break;

	case ID_PRINTALL    :
          CardFilePrint();
          break;

	case ID_EXIT        :
          PostQuitMessage(0);
          break;

	case ID_ABOUT       :
          MessageBox("MEWEL Window System", "Magma Systems", "(201) 912-0192",
                     "About CardFile", MB_OK);
          break;

	case ID_CUT         :
          if (hActiveEdit)
            SendMessage(hActiveEdit, WM_CUT, 0, 0L);
          break;

	case ID_UNDO        :
          if (hActiveEdit)
          {
            if (SendMessage(hActiveEdit, EM_CANUNDO, 0, 0L))
              SendMessage(hActiveEdit, EM_UNDO, 0, 0L);
            else
              MessageBox("No undo capability", "for this edit field", 
                         NULL, "Undo", MB_OK);
          }
          break;

	case ID_COPY        :
          if (hActiveEdit)
            SendMessage(hActiveEdit, WM_COPY, 0, 0L);
          break;

	case ID_PASTE       :
          if (hActiveEdit)
            SendMessage(hActiveEdit, WM_PASTE, 0, 0L);
          break;

	case ID_RESTORE     :
          MessageBox("Not implemented", "in this version", 
                         NULL, "Restore", MB_OK);
          break;

	case ID_INDEX       :
          rc = GenericDialogDriver(DLG_INDEX, IndexDialogProc);
          if (rc == IDOK)
          {
            InvalidateRect(GetWindowWord(hCurrCard, GWW_HWNDINDEX), (LPRECT) NULL, FALSE);
            UpdateWindow(GetWindowWord(hCurrCard, GWW_HWNDINDEX));
          }
          break;

	case ID_ADD         :
          CardCreate();
          break;
	case ID_DELETE      :
          CardDelete();
          break;
	case ID_DUPLICATE   :
          CardDuplicate();
          break;
	case ID_DIAL        :
          rc = GenericDialogDriver(DLG_DIAL, DialDialogProc);
          break;

	case ID_VIEWCARD    :
          fCardfileState &= ~STATE_VIEWLIST;
          CheckMenuItem(GetMenu(hWndCardfile), ID_VIEWCARD, MF_CHECKED | MF_BYCOMMAND);
          CheckMenuItem(GetMenu(hWndCardfile), ID_VIEWLIST, MF_UNCHECKED | MF_BYCOMMAND);
          CardFileShowCards(TRUE);
          CardFileAdjustWindows();
          break;
	case ID_VIEWLIST    :
          fCardfileState |= STATE_VIEWLIST;
          CheckMenuItem(GetMenu(hWndCardfile), ID_VIEWCARD, MF_UNCHECKED | MF_BYCOMMAND);
          CheckMenuItem(GetMenu(hWndCardfile), ID_VIEWLIST, MF_CHECKED | MF_BYCOMMAND);
          CardFileShowCards(FALSE);
          CardFileAdjustWindows();
          SetFocus(hWnd);
          break;

	case ID_GOTO        :
          rc = GenericDialogDriver(DLG_GOTO, GotoDialogProc);
          if (rc >= 1 && rc <= CurrCardFile.nRecords)
          {
            CurrCardFile.iCurrRecord = rc-1;
            CardFileAdjustWindows();
          }
          break;
	case ID_FIND        :
          rc = GenericDialogDriver(DLG_FIND, FindDialogProc);
          if (rc == IDCANCEL)
            break;
          /* fall through ... */
	case ID_FINDNEXT    :
          CardFileSearch(szSearchString);
          break;

        case ID_FORWARD     :
          if (CurrCardFile.iCurrRecord < CurrCardFile.nRecords - 1)
            CurrCardFile.iCurrRecord++;
          else
            CurrCardFile.iCurrRecord = 0;
          CardFileAdjustWindows();
          break;
        case ID_BACKWARD    :
          if (CurrCardFile.iCurrRecord > 0)
            CurrCardFile.iCurrRecord--;
          else
            CurrCardFile.iCurrRecord = CurrCardFile.nRecords - 1;
          CardFileAdjustWindows();
          break;
        case ID_FIRSTTOFRONT:
          CurrCardFile.iCurrRecord = 0;
          CardFileAdjustWindows();
          break;
        case ID_LASTTOFRONT :
          CurrCardFile.iCurrRecord = CurrCardFile.nRecords - 1;
          CardFileAdjustWindows();
          break;

	case ID_HELP        :
          goto dohelp;
      }
      break;

    case WM_SYSCOMMAND :
      switch (wParam)
      {
        case SC_CLOSE :
          PostQuitMessage(0);
          break;
        default :
          return DefWinProc(hWnd, message, wParam, lParam);
      }
      break;


  case WM_VSCROLL :
    switch (wParam)
    {
      case SB_LINEUP         :
      case SB_PAGEUP         :
        PostMessage(hWnd, WM_COMMAND, ID_BACKWARD, 0L);
        break;
      case SB_LINEDOWN       :
      case SB_PAGEDOWN       :
        PostMessage(hWnd, WM_COMMAND, ID_FORWARD, 0L);
        break;
      case SB_THUMBTRACK :
      {
        HWND hSB = HIWORD(lParam);
        int  minpos, maxpos, currpos;
        int  oldCurrSel;

        GetScrollRange(hSB, SB_CTL, &minpos, &maxpos);
        currpos = LOWORD(lParam);
        SetScrollPos(hSB, SB_CTL, max(min(currpos, maxpos), minpos), FALSE);
        currpos = GetScrollPos(hSB, SB_CTL);
        CurrCardFile.iCurrRecord = currpos-1;
        CardFileAdjustWindows();
      }
      break;
    }

    break;


  case WM_LBUTTONDBLCLK:
  case WM_LBUTTONDOWN :
    if (fCardfileState & STATE_VIEWLIST)
    {
      int index;
      RECT rClient;

      GetClientRect(hWnd, &rClient);
      index = HIWORD(lParam) - rClient.top;
      if (index < 0 || index >= rClient.bottom)
        break;
      CurrCardFile.iCurrRecord = index;
      CardFileAdjustWindows();

      /*
        If we double-clicked on an entry, bring up the index dialog box
      */
      if (message == WM_LBUTTONDBLCLK)
        PostMessage(hWnd, WM_COMMAND, ID_INDEX, 0L);
    }
    goto callDWP;


  case WM_CHAR :
    switch (wParam)
    {
      case VK_HOME :
        if (fCardfileState & STATE_VIEWLIST)
          PostMessage(hWnd, WM_COMMAND, ID_FIRSTTOFRONT, 0L);
        break;
      case VK_END  :
        if (fCardfileState & STATE_VIEWLIST)
          PostMessage(hWnd, WM_COMMAND, ID_LASTTOFRONT, 0L);
        break;
      case VK_DOWN :
        if (fCardfileState & STATE_VIEWLIST)
          PostMessage(hWnd, WM_COMMAND, ID_FORWARD, 0L);
        break;
      case VK_UP   :
        if (fCardfileState & STATE_VIEWLIST)
          PostMessage(hWnd, WM_COMMAND, ID_BACKWARD, 0L);
        break;
      case VK_PGUP :
        if (fCardfileState & STATE_VIEWLIST)
        {
          RECT rClient;
          GetClientRect(hWndCardfile, &rClient);
          CurrCardFile.iCurrRecord = max(0, 
                         CurrCardFile.iCurrRecord - RECT_HEIGHT(rClient));
          CardFileAdjustWindows();
        }
        else
          PostMessage(hWnd, WM_COMMAND, ID_BACKWARD, 0L);
        break;
      case VK_PGDN :
        if (fCardfileState & STATE_VIEWLIST)
        {
          RECT rClient;
          GetClientRect(hWndCardfile, &rClient);
          CurrCardFile.iCurrRecord = min(CurrCardFile.nRecords-1, 
                          CurrCardFile.iCurrRecord + RECT_HEIGHT(rClient));
          CardFileAdjustWindows();
        }
        else
          PostMessage(hWnd, WM_COMMAND, ID_FORWARD, 0L);
        break;
      default      :   /* CTRL+letter */
        if (wParam > 0 && wParam < 32)
          CardFileSearchForLetter(wParam + 'A' - 1);
        break;
    }
    break;


  case WM_HELP :
dohelp:
    GenericDialogDriver(DLG_HELP, HelpDialogProc);
    break;

 default :
callDWP:
   /* Call the default window procedure for the main window */
   return DefWinProc(hWnd, message, wParam, lParam);
  }
}




long PASCAL CardWndProc(hWnd, message, wParam, lParam)
  HWND hWnd;
  WORD message;
  WORD wParam;
  DWORD lParam;
{
  INT  rc;
  HWND hEditIndex, hEditCard;
  RECT r;

  switch (message)
  {
    case WM_CREATE :
      GetWindowRect(hWnd, (LPRECT) &r);

      hEditIndex = CreateWindow(
                      "Normal",  /* "Edit" */
                       NULL,           /* title */
                       WS_BORDER | WS_CAPTION,
                       r.left, r.top,
                       RECT_WIDTH(r), /*3*/ RECT_HEIGHT(r), /* x,y,width,height */
                       SYSTEM_COLOR,   /* attr */
                       0,              /* id */
                       hWnd,           /* hParent */
                       NULLHWND,       /* hMenu */
                       0,              /* hInst */
                       0L);            /* lpParam */
      WinSetWinProc(hEditIndex, IndexEditWndProc);

      hEditCard  = CreateWindow(
                      "Edit",
                       NULL,           /* title */
                       /*ES_MARGIN | */ES_MULTILINE,
                       r.left+1, /*r.top+2*/r.top+1,
                       RECT_WIDTH(r)-2, RECT_HEIGHT(r) - 2,
                       SYSTEM_COLOR,   /* attr */
                       0,              /* id */
                       hWnd,           /* hParent */
                       NULLHWND,       /* hMenu */
                       0,              /* hInst */
                       0L);            /* lpParam */
      WinSetWinProc(hEditCard, CardEditWndProc);

      SetWindowWord(hWnd, GWW_HWNDINDEX, hEditIndex);
      SetWindowWord(hWnd, GWW_HWNDEDIT,  hEditCard);
      break;

    case WM_SETFOCUS :
      SetFocus((HWND) GetWindowWord(hWnd, GWW_HWNDEDIT));
      break;

    default :
      /* Call the default window procedure for the main window */
      return DefWinProc(hWnd, message, wParam, lParam);
  }
}

long PASCAL CardEditWndProc(hWnd, message, wParam, lParam)
  HWND hWnd;
  WORD message;
  WORD wParam;
  DWORD lParam;
{
  WORD attr;
  RECT r;

  switch (message)
  {
    case WM_BORDER :
      return FALSE;
      attr = WinGetAttr(hWnd);
      WinDrawFrame(hWnd, attr, GetFocus() == hWnd);
      GetWindowRect(hWnd, &r);
      WinPutc(hWnd, -1, -1, (GetFocus() == hWnd) ? '' : '', attr);
      WinPutc(hWnd, -1, RECT_WIDTH(r), (GetFocus() == hWnd) ? '' : '', attr);
      return TRUE;

    case WM_SETFOCUS :
      hActiveEdit = hWnd;
      CardBringToTop(GetParent(hWnd));
      return DefWinProc(hWnd, message, wParam, lParam);

    default :
      /* Call the default window procedure for the main window */
      return DefWinProc(hWnd, message, wParam, lParam);
  }
}

long PASCAL IndexEditWndProc(hWnd, message, wParam, lParam)
  HWND hWnd;
  WORD message;
  WORD wParam;
  DWORD lParam;
{
  WORD attr;
  RECT r;

  switch (message)
  {
    case WM_BORDER :
      return FALSE;
      attr = WinGetAttr(hWnd);
      WinDrawFrame(hWnd, attr, GetFocus() == hWnd);
      GetWindowRect(hWnd, &r);
      WinPutc(hWnd, 1, -1, (GetFocus() == hWnd) ? '' : '', attr);
      WinPutc(hWnd, 1, RECT_WIDTH(r), (GetFocus() == hWnd) ? '' : '', attr);
      return TRUE;

    case WM_SETFOCUS :
      hActiveEdit = hWnd;
      CardBringToTop(GetParent(hWnd));
      return DefWinProc(hWnd, message, wParam, lParam);

    default :
      /* Call the default window procedure for the main window */
      return DefWinProc(hWnd, message, wParam, lParam);
  }
}


